Index: src/drivers/X11/Fl_X11_Screen_Driver.cxx
===================================================================
--- src/drivers/X11/Fl_X11_Screen_Driver.cxx	(Revision 12239)
+++ src/drivers/X11/Fl_X11_Screen_Driver.cxx	(Arbeitskopie)
@@ -30,6 +30,7 @@
 #include <FL/Fl_Tooltip.H>
 
 #include <sys/time.h>
+#include <math.h>
 
 #if HAVE_XINERAMA
 #  include <X11/extensions/Xinerama.h>
@@ -779,6 +780,10 @@
 		green_shift,
 		blue_mask,
 		blue_shift;
+
+  timeval start, end;
+  gettimeofday( &start, 0 );
+
   //
   // Under X11 we have the option of the XGetImage() interface or SGI's
   // ReadDisplay extension which does all of the really hard work for
@@ -848,6 +853,10 @@
   }
   
   if (!image) return 0;
+
+  gettimeofday( &end, 0 );
+  printf( "get XImage (%dx%d): %lu ms\n", (int)(w*s), (int)(h*s), lround( 0.001 * ((end.tv_sec-start.tv_sec) * 1000000 + (end.tv_usec-start.tv_usec))) );
+
   if (s != 1) {
     w = w*s < 1 ? 1 : int(w*s);
     h = h*s < 1 ? 1 : int(h*s);
@@ -877,7 +886,61 @@
   // Allocate the image data array as needed...
   const uchar *oldp = p;
   if (!p) p = new uchar[w * h * d];
+
+#if 1 // disable patch with #if 0 to measure original speed
+  // Speed up the most common conversion of
+  // RGB XImage d=3/4 and r/g/b values in full bytes,
+  // avoiding costly shift/multiply/divide operations
+  // => about 10x faster.
+  if (!image->red_mask &&
+      fl_visual->red_mask   == 0x00ff0000 &&
+      fl_visual->green_mask == 0x0000ff00 &&
+      fl_visual->blue_mask  == 0x000000ff &&
+      image->bits_per_pixel == 32) {
+
+    gettimeofday( &start, 0 );
+
+    pixel = (unsigned char *)image->data;
+    unsigned char *drgb = p;
+    unsigned char *e = drgb + w * h * d;
+    // Initialize the default colors/alpha in the whole image...
+    if (d==4)
+      memset(drgb, alpha, w * h * d);
+
+    int delta = d == 4 ? 1 : 0;
+    if ( image->byte_order == LSBFirst ) {
+      while (drgb < e) {
+        *drgb++ = pixel[2];
+        *drgb++ = pixel[1];
+        *drgb++ = pixel[0];
+        drgb += delta;
+        pixel += 4;
+      }
+    }
+    else { // MSBFirst
+      while (drgb < e) {
+        *drgb++ = *pixel++;
+        *drgb++ = *pixel++;
+        *drgb++ = *pixel++;
+        drgb += delta;
+        pixel ++;
+      }
+    }
+
+    gettimeofday( &end, 0 );
+    printf( "copy XImage (%dx%d): %lu ms\n", w, h, lround( 0.001 * ((end.tv_sec-start.tv_sec) * 1000000 + (end.tv_usec-start.tv_usec))) );
+
+    // Destroy the X image we've read and return the RGB(A) image...
+    XDestroyImage(image);
   
+    Fl_RGB_Image *rgb = new Fl_RGB_Image(p, w, h, d);
+    if (!oldp) rgb->alloc_array = 1;
+    return rgb;
+  }
+#endif
+
+  gettimeofday( &start, 0 );
+
   // Initialize the default colors/alpha in the whole image...
   memset(p, alpha, w * h * d);
   
@@ -1172,6 +1235,9 @@
     }
   }
   
+  gettimeofday( &end, 0 );
+  printf( "copy XImage (%dx%d): %lu ms\n", w, h, lround( 0.001 * ((end.tv_sec-start.tv_sec) * 1000000 + (end.tv_usec-start.tv_usec))) );
+
   // Destroy the X image we've read and return the RGB(A) image...
   XDestroyImage(image);
   
